try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse


def assert_url_equal(expected, returned):
    parsed_expected = urlparse.urlparse(expected)
    parsed_returned = urlparse.urlparse(returned)

    error = '{0}: "{1}" did not match expected {0}: "{2}"'
    for attr in ['scheme', 'netloc', 'path', 'params', 'fragment']:
        parsed = getattr(parsed_expected, attr)
        expect = getattr(parsed_returned, attr)
        assert parsed == expect, error.format(attr, expect, parsed)

    parsed_query = urlparse.parse_qs(parsed_expected.query)
    expected_query = urlparse.parse_qs(parsed_returned.query)

    assert expected_query == parsed_query, error.format(
        'query', expected_query, parsed_query)



from __future__ import absolute_import

from test_helpers import url
from test_helpers.compat import unittest


#########
#
# url.assert_url_equal
#
#########


class WhenComparingURLs(unittest.TestCase):

    def should_not_explode_when_the_args_differ_in_order(self):
        url.assert_url_equal(
            'http://www.google.com?q=foo&arg=baz',
            'http://www.google.com?arg=baz&q=foo',
        )



from __future__ import absolute_import

from tornado import web

from test_helpers import bases, mixins
from test_helpers.mixins import tornado


class RecordingRequestHandler(web.RequestHandler):

    def on_request(self, *args, **kwargs):
        self.application.record_request(self.request)
        self.set_status(200)

    delete = on_request
    get = on_request
    head = on_request
    options = on_request
    post = on_request
    put = on_request


class Recorder(web.Application):

    def __init__(self):
        super(Recorder, self).__init__(handlers=[
            ('/', RecordingRequestHandler),
        ])
        self.requests = []

    def record_request(self, request):
        self.requests.append(request)


class _TornadoMixinTestCase(mixins.PatchMixin, bases.BaseTest):

    @classmethod
    def configure(cls):
        super(_TornadoMixinTestCase, cls).configure()
        cls.application = Recorder()
        cls.test_instance = tornado.TornadoMixin
        cls.test_instance.start_tornado(cls.application)

    @classmethod
    def annihilate(cls):
        super(_TornadoMixinTestCase, cls).annihilate()
        cls.test_instance.stop_tornado()

    @property
    def request(self):
        return self.application.requests[0]

    def should_issue_single_request(self):
        self.assertEqual(len(self.application.requests), 1)


class WhenTornadoMixinGets(_TornadoMixinTestCase):

    @classmethod
    def execute(cls):
        cls.test_instance.get('/')

    def should_get_from_request_handler(self):
        self.assertEqual(self.request.method, 'GET')

    def should_get_requested_path(self):
        self.assertEqual(self.request.path, '/')


class WhenTornadoMixinPosts(_TornadoMixinTestCase):

    @classmethod
    def execute(cls):
        cls.test_instance.post('/', 'body')

    def should_post_to_request_handler(self):
        self.assertEqual(self.request.method, 'POST')

    def should_post_to_requested_path(self):
        self.assertEqual(self.request.path, '/')

    def should_post_body(self):
        self.assertEqual(self.request.body, b'body')


class WhenTornadoMixinDeletes(_TornadoMixinTestCase):

    @classmethod
    def execute(cls):
        cls.test_instance.delete('/')

    def should_send_delete_to_request_handler(self):
        self.assertEqual(self.request.method, 'DELETE')

    def should_delete_requested_resource(self):
        self.assertEqual(self.request.path, '/')


class WhenTornadoMixinPuts(_TornadoMixinTestCase):

    @classmethod
    def execute(cls):
        cls.test_instance.put('/', 'body')

    def should_put_to_request_handler(self):
        self.assertEqual(self.request.method, 'PUT')

    def should_put_to_requested_path(self):
        self.assertEqual(self.request.path, '/')

    def should_put_body(self):
        self.assertEqual(self.request.body, b'body')


class WhenTornadoMixinRequestsOptions(_TornadoMixinTestCase):

    @classmethod
    def execute(cls):
        cls.test_instance.request('OPTIONS', '/')

    def should_request_options__from_request_handler(self):
        self.assertEqual(self.request.method, 'OPTIONS')

    def should_request_appropriate_resource(self):
        self.assertEqual(self.request.path, '/')



"""Collection of functionality mix-ins.

This module contains standalone classes that can be safely mixed
into the :class:`~test_helpers.bases.BaseTest` class.  Each mixin
extends the functionality of the test case by adding behaviors,
methods, and attributes - for example, patching well-understood
functionality or automatically creating/destroying an in memory
UDP server.

When creating a mixin it is important to take not that you
should strive to keep the Method Resolution Order (MRO) as clean
as possible.  Each mixin class should ideally only inherit from
``object``.

"""

from .patch_mixin import PatchMixin
from .tornado import TornadoMixin

__all__ = ('PatchMixin', 'TornadoMixin')



"""
Testing Utilities
=================

The testing utilities module contains standalone functionality for that might
be useful for a select number of test cases.  These functions can be
selectively applied to a small subset of tests so they might not warrant the
full capacity of mixin behavior.

The utilities within this module are typically simple functions or decorators
that ease a specific testing task, such as creating patches.

"""

import functools

from test_helpers.compat import mock


def create_ppatch(path):
    """Create a partial ppatch object that will only require the object name"""
    return functools.partial(ppatch, path)


def ppatch(path, object_name, **kwargs):
    """Creates a fully qualified patch object.

    This function will act as a wrapper that will allow us to create a partial
    function representation.  That will remove the need to keep passing the
    same path to the patch object.

    """
    return mock.patch(path.format(object_name), **kwargs)



from __future__ import absolute_import

from tornado import httpclient

from test_helpers.compat import mock
from test_helpers import bases, mixins
from test_helpers.mixins import tornado


class _JsonMixinTestCase(mixins.PatchMixin, bases.BaseTest):
    patch_prefix = 'test_helpers.mixins.tornado'

    @classmethod
    def configure(cls):
        cls.real_response = httpclient.HTTPResponse(
            request=mock.Mock(), code=200, buffer=mock.Mock())
        super_lookup = cls.create_patch('super', create=True)
        cls.super_request = super_lookup.return_value.request
        cls.super_request.return_value = cls.real_response

        cls.mixin = tornado.JsonMixin()
        cls.request_kwargs = {}

    @classmethod
    def execute(cls):
        cls.response = cls.mixin.request(
            mock.sentinel.method, **cls.request_kwargs)

    def should_return_response_object(self):
        self.assertIs(self.response, self.real_response)


class WhenJsonMixinRequestsWithoutBody(_JsonMixinTestCase):

    def should_call_super_request_with_accept_header(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            headers={'Accept': 'application/json'},
        )

    def should_set_json_to_none_on_response_object(self):
        self.assertIsNone(self.response.json)


class WhenJsonMixinRequestsWithJsonBody(_JsonMixinTestCase):

    @classmethod
    def configure(cls):
        super(WhenJsonMixinRequestsWithJsonBody, cls).configure()
        cls.request_kwargs['body'] = {'foo': 'bar'}

    def should_use_jsonified_body_in_request(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            body='{"foo": "bar"}'.encode('utf-8'),
            headers=mock.ANY,
        )

    def should_use_appropriate_headers(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            body=mock.ANY,
            headers={
                'Content-Type': 'application/json; charset=utf-8',
                'Accept': 'application/json',
            },
        )


class WhenJsonMixinRequestsWithJsonBodyAndCustomMimeType(
        WhenJsonMixinRequestsWithJsonBody):

    @classmethod
    def configure(cls):
        super(WhenJsonMixinRequestsWithJsonBodyAndCustomMimeType, cls).configure()
        cls.request_kwargs['headers'] = {
            'content-type': 'application/vnd.aweber.com-login+json',
        }

    def should_use_appropriate_headers(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            body=mock.ANY,
            headers={
                'Content-Type': 'application/vnd.aweber.com-login+json',
                'Accept': 'application/json',
            },
        )


class WhenJsonMixinRequestsWithNonJsonBody(_JsonMixinTestCase):

    @classmethod
    def configure(cls):
        super(WhenJsonMixinRequestsWithNonJsonBody, cls).configure()
        cls.request_kwargs['body'] = mock.sentinel.body
        cls.request_kwargs['headers'] = {
            'content-type': 'application/x-www-form-urlencoded',
        }

    def should_pass_unaltered_body(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            body=mock.sentinel.body,
            headers=mock.ANY,
        )

    def should_use_appropriate_headers(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            body=mock.ANY,
            headers={
                'Content-Type': 'application/x-www-form-urlencoded',
                'Accept': 'application/json',
            }
        )


class WhenJsonMixinRequestsWithCustomAcceptHeader(_JsonMixinTestCase):

    @classmethod
    def configure(cls):
        super(WhenJsonMixinRequestsWithCustomAcceptHeader, cls).configure()
        cls.request_kwargs['headers'] = {'accept': 'text/html'}

    def should_pass_custom_accept_header(self):
        self.super_request.assert_called_once_with(
            mock.sentinel.method,
            headers={'Accept': 'text/html'},
        )


class WhenJsonMixinGetsJsonResponse(_JsonMixinTestCase):

    @classmethod
    def configure(cls):
        super(WhenJsonMixinGetsJsonResponse, cls).configure()
        cls.json_loads = cls.create_patch('json').loads
        cls.real_response.headers['content-type'] = (
            'application/json; charset=utf8')
        cls.real_response._body = b'{"body":"value"}'

    def should_json_loads_response_body(self):
        self.json_loads.assert_called_once_with('{"body":"value"}')

    def should_save_json_on_response_object(self):
        self.assertIs(
            self.response.json, self.json_loads.return_value)


class WhenJsonMixinGetsNonJsonResponse(_JsonMixinTestCase):

    @classmethod
    def configure(cls):
        super(WhenJsonMixinGetsNonJsonResponse, cls).configure()
        cls.real_response.headers['content-type'] = 'text/plain'

    def should_set_json_to_none_on_response_object(self):
        self.assertIsNone(self.response.json)



from __future__ import absolute_import

from test_helpers.bases import BaseTest
from test_helpers import utils

ppatch = utils.create_ppatch('tests.integration.test_helpers')


#########
#
# mixins.PatchMixin
#
#########


class WhenCreatingAPartialPatch(BaseTest):

    @classmethod
    @ppatch('function_under_test', return_value='foobar')
    def configure(cls, mocked):
        cls.mocked = mocked

    @classmethod
    def execute(cls):
        cls.result = cls.mocked()

    def should_have_called_the_mocked_function(self):
        self.assertEqual(self.result, 'foobar')



from __future__ import absolute_import

import json
import os
import socket

from six import text_type
from tornado import ioloop, httpclient, httpserver, httputil

from .. import bases, compat


class TornadoMixin(object):

    """Test tornado applications with AAA testing.

    Mix this class in over :class:`test_helpers.bases.BaseTest` or
    similar classmethod-based testing class and you can directly
    test tornado-based applications.

    **Usage**

    .. code-block:: python

       from unittest import TestCase
       import test_helpers.mixins.tornado
       import myproject

       class WhenMyApplicationsGets(mixins.tornado.TornadoMixin, TestCase):

           @classmethod
           def setUpClass(cls):
               super(WhenMyApplicationGets, cls).setUpClass()
               cls.start_tornado(myproject.Application())
               cls.execute()

           @classmethod
           def tearDownClass(cls):
               super(WhenMyApplicationGets, cls).tearDownClass()
               cls.stop_tornado()

           @classmethod
           def execute(cls):
               cls.response = cls.get('/index')

           def should_return_ok(self):
               self.assertEqual(self.response.code, 200)

    **Attributes**

    This mix-in creates three useful attributes in addition to the
    methods.  Each of the attributes is initialized by :meth:`start_tornado`
    and will be :data:`None` until that method is called.

    .. attribute:: client

       The :class:`tornado.httpclient.HTTPClient` instance used to
       interact with the application.

    .. attribute:: io_loop

       The :class:`tornado.ioloop.IOLoop` instance that the application
       is attached to.

    .. attribute:: url_root

       The URL root used to interact with the application.  This refers
       to host and port that :attr:`io_loop` is attached to.

    .. attribute:: request_timeout

       The number of seconds to run the IO loop for before giving up
       on the request.

    """

    client = None
    io_loop = None
    request_timeout = float(os.environ.get('ASYNC_TEST_TIMEOUT', '20.0'))
    url_root = None
    _result = None
    _request_timeout_failure = object()

    @classmethod
    def start_tornado(cls, application):
        """Start up tornado and register `application`.

        :param application: anything suitable as a Tornado *request
            callback* (see :class:`tornado.httpserver.HTTPServer`)

        This method binds a socket to an arbitrary temporary port,
        creates a new Tornado IOLoop, and adds the `application`'
        to it.  Calling any of the request-related methods will
        start the IOLoop and run it until a response is received.

        """
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        sock.setblocking(0)
        sock.bind(('127.0.0.1', 0))
        sock.listen(128)

        cls.url_root = 'http://{0}:{1}'.format(*sock.getsockname())
        cls.io_loop = ioloop.IOLoop()
        cls.io_loop.make_current()

        cls.client = httpclient.AsyncHTTPClient(io_loop=cls.io_loop)
        cls.server = httpserver.HTTPServer(application, cls.io_loop)
        cls.server.add_socket(sock)

    @classmethod
    def stop_tornado(cls):
        """Terminate the tornado IO loop."""
        cls.io_loop.clear_current()
        cls.io_loop.close(all_fds=True)

    @classmethod
    def delete(cls, path, **kwargs):
        """Issue a ``DELETE`` request."""
        return cls.request('DELETE', path, **kwargs)

    @classmethod
    def get(cls, path, **kwargs):
        """Issue a ``GET`` request."""
        return cls.request('GET', path, **kwargs)

    @classmethod
    def post(cls, path, body, **kwargs):
        """Issue a ``POST`` request."""
        return cls.request('POST', path, body=body, **kwargs)

    @classmethod
    def put(cls, path, body, **kwargs):
        """Issue a ``PUT`` request."""
        return cls.request('PUT', path, body=body, **kwargs)

    @classmethod
    def request(cls, method, path, **kwargs):
        """Issue a request to the application.

        :param str method: HTTP method to invoke
        :param str path: possibly absolute path of the resource
            to invoke
        :param kwargs: additional arguments passed to the
            :class:`tornado.httpclient.HTTPRequest` initializer

        :returns: either a :class:`tornado.httpclient.HTTPResponse`
            instance or :data:`None` if the request timed out.

        The `path` parameter can be a relative path or an absolute
        URL.  It will be joined to :attr:`url_root` before the
        request object is created.

        """
        request = httpclient.HTTPRequest(
            compat.urljoin(cls.url_root, path), method=method, **kwargs)
        cls.client.fetch(request, callback=cls._on_request_complete)
        return cls._wait_for_completion()

    @classmethod
    def _on_request_complete(cls, result, **_):
        """Called when a request completes or times out.

        :param result: the response received from the application
            or :data:`._request_timeout_failure`

        """
        cls._result = result
        cls.io_loop.stop()

    @classmethod
    def _wait_for_completion(cls):
        def timeout():
            try:
                raise RuntimeError('timeout failure')
            finally:
                cls._on_request_complete(cls._request_timeout_failure)

        tmo_handle = cls.io_loop.add_timeout(
            cls.io_loop.time() + cls.request_timeout, timeout)
        cls.io_loop.start()
        cls.io_loop.remove_timeout(tmo_handle)

        assert cls._result is not cls._request_timeout_failure
        return cls._result


class TornadoTest(TornadoMixin, bases.BaseTest):

    """Tornado version of :class:`~.bases.BaseTest`.

    This class acts as a replacement for :class:`~.bases.BaseTest`
    with the Tornado magic pre-mixed.  All that you have to do
    is:

    1. set :attr:`tornado_application` as a top-level class
       attribute *OR*
    2. pass the ``application`` keyword to :meth:`configure`
       and make sure that this class is the first one in the
       ``__mro__``

    In either case, the ``application`` is the Tornado *request
    callback* that is being tested.  See
    :class:`tornado.httpserver.HTTPServer` for a complete
    description of what constitutes a *"request callback"*.

    """

    tornado_application = None
    """The Tornado request handler callable."""

    @classmethod
    def configure(cls, application=None):
        """Configures the Tornado IOLoop.

        :param application: overrides :attr:`tornado_application`

        """
        cls.start_tornado(application or cls.tornado_application)
        super(TornadoTest, cls).configure()

    @classmethod
    def annihilate(cls):
        """Terminates the Tornado application."""
        super(TornadoTest, cls).annihilate()
        cls.stop_tornado()


class JsonMixin(object):

    """Mix in over :class:`TornadoMixin` to enable JSON handling.

    This mix in extends :meth:`TornadoMixin.request` so that it will
    automatically serialize request data and deserialize responses
    as JSON.  It does honor the HTTP content type headers so it will
    not accidentally deserialize a non-JSON response or serialize an
    already serialized request.

    """

    @classmethod
    def request(cls, *args, **kwargs):
        """Send a request and process the response.

        :param args: positional parameters to send to ``super().request``
        :param kwargs: keyword parameters to send to ``super().request``

        :returns: either a :class:`tornado.httpclient.HTTPResponse`
            instance or :data:`None` if the request timed out.

        """
        headers = httputil.HTTPHeaders()
        if 'headers' in kwargs:
            headers.update(kwargs.pop('headers'))

        if 'body' in kwargs:
            header_val = headers.get('content-type', 'application/json')
            content_type = header_val
            if ';' in content_type:
                content_type, _ = header_val.split(';', 1)
            body = kwargs.get('body')
            if (content_type.endswith('json')
                    and not isinstance(body, text_type)):
                kwargs['body'] = json.dumps(body).encode('utf-8')
                if 'content-type' not in headers:
                    content_type = 'application/json; charset=utf-8'
                    headers['Content-Type'] = content_type

        if 'accept' not in headers:
            headers['Accept'] = 'application/json'

        if headers:
            kwargs['headers'] = headers

        response = super(JsonMixin, cls).request(*args, **kwargs)
        response.json = None

        header_val = response.headers.get(
            'content-type', 'application/octet-stream')
        content_type, params = header_val, ''
        if ';' in content_type:
            content_type, params = header_val.split(';', 1)
        if content_type.endswith('json'):
            encoding = 'utf-8'
            for param in params.split(','):
                if param.strip().startswith('charset='):
                    encoding = param.split('=')[1]
            try:
                response.json = json.loads(response.body.decode(encoding))
            except:
                response.json = None

        return response



from .. compat import mock


class PatchMixin(object):
    """A mixin to allow inline patching and automatic un-patching.

    This mixin adds one new method, ``create_patch`` that will create and
    activate patch objects without having to use the decorator.

    In order to make use of the patching functionality you need to set
    the ``patch_prefix`` class attribute.  This attribute should be the python
    module path whose objects you want to patch.  For example, if you wanted
    to patch the ``baz`` object in the ``foo.bar`` module your patch prefix
    might look like ``foo.bar``.  When creating a patch you can now just refer
    to the object name like ``cls.create_patch('baz')``.

    This usage of this mixin as opposed to the patch decorator results
    in less pylint errors and not having to think about the order of decorator
    application.

    Example Usage::

        class MyTest(mixins.PatchMixin, bases.BaseTest):

            patch_prefix = 'my_application.module.submodule'

            @classmethod
            def configure(cls):
                cls.foo_mock = cls.create_patch('foo')
                cls.bar_mock = cls.create_patch('bar', return_value=100)

            @classmethod
            def execute(cls):
                function_under_test()

            def should_call_foo(self):
                self.foo_mock.assert_called_once_with()

            def should_return_100_from_bar(self):
                self.assertEqual(100, self.bar_mock.return_value)

    """

    patch_prefix = ''
    """Pattern used to generate fully-qualified patch names."""

    @classmethod
    def setUpClass(cls):
        cls._active_patches = []
        super(PatchMixin, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.stop_patches()
        super(PatchMixin, cls).tearDownClass()

    @classmethod
    def stop_patches(cls):
        """Stop any active patches when the class is finished."""
        for patcher in cls._active_patches:
            patcher.stop()
        cls._active_patches = []

    @classmethod
    def create_patch(cls, target, **kwargs):
        """Create and apply a patch.

        This method calls :func:`mock.patch` with the keyword parameters
        and returns the running patch.  This approach has the benefit of
        applying a patch without scoping the patched code which, in turn,
        lets you apply patches without having to override :meth:`setUpClass`
        to do it.

        :param str target: the target of the patch.  This is passed as
            an argument to ``cls.patch_prefix.format()`` to create the
            fully-qualified patch target.

        """
        patch_prefix = cls.patch_prefix

        if patch_prefix and not patch_prefix.endswith('.'):
            patch_prefix += '.'

        path = '{0}{1}'.format(patch_prefix, target)
        patcher = mock.patch(path, **kwargs)
        patched = patcher.start()
        cls._active_patches.append(patcher)
        return patched



from __future__ import absolute_import

import socket

from test_helpers.bases import BaseTest
from test_helpers import mixins


#########
#
# mixins.PatchMixin
#
#########


class WhenTestingPatchingMixin(mixins.PatchMixin, BaseTest):

    patch_prefix = 'tests.integration.test_mixins'

    @classmethod
    def configure(cls):
        cls.mocked = cls.create_patch('function_under_test', create=True)
        cls.mocked.return_value = 'foobar'

    @classmethod
    def execute(cls):
        cls.result = cls.mocked()

    def should_have_called_the_mocked_function(self):
        self.assertEqual(self.result, 'foobar')



"""
===============
Base Test Cases
===============

The base test cases module contains useful classes that inherit from the
unit test standard library and optionally inherit from an arbitrary number of
mixins.  The purpose of these classes is to provide an integration point for
the mixins and to help promote the usage of the Arrange-Act-Assert testing
methodology used here at AWeber.

"""
from test_helpers.compat import unittest


class BaseTest(unittest.TestCase):
    """Base class for the AWeber AAA testing style.

    This implements the Arrange-Act-Assert style of unit testing though
    the names were chosen to match existing convention.  New unit tests
    should use this as a base class and replace the :meth:`configure` and
    :meth:`execute` methods as necessary.

    """

    maxDiff = 100000

    @classmethod
    def setUpClass(cls):
        """Arrange the test and do the action.

        If you are extending this method, then you are required to call
        this implementation as the last thing in your version of this
        method.

        """
        super(BaseTest, cls).setUpClass()
        try:
            cls.configure()
            cls.execute()
        except:
            cls.annihilate()
            raise

    @classmethod
    def tearDownClass(cls):
        super(BaseTest, cls).tearDownClass()
        cls.annihilate()

    @classmethod
    def configure(cls):
        """Extend to configure your test environment."""
        pass

    @classmethod
    def annihilate(cls):
        """Clean up after a test.

        Unlike :meth:`tearDownClass`, this method is guaranteed to
        be called in all cases.  It will be called even if
        :meth:`configure` fails so do not do anything that depends
        on it having been successful without checking if it was.

        """
        pass

    @classmethod
    def execute(cls):
        """Override to execute your test action."""
        raise NotImplementedError('The execute action was not defined!')



import sys

if sys.version_info >= (2, 7):  # pragma: no cover
    import unittest
else:
    import unittest2 as unittest

try:
    from unittest import mock
except ImportError:
    import mock

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


__all__ = (
    'mock',
    'unittest',
    'urljoin',
)



#!/usr/bin/env python
import codecs

from setuptools import setup, find_packages

VERSION = '1.5.0'

def read_requirements_file(filename):
    """Read pip-formatted requirements from a file."""
    with open(filename, 'r') as f:
        return [
            line.strip() for line in f.readlines()
            if not line.startswith('#')
        ]

setup(
    name='test-helpers',
    description='A collection of test helpers to facilitate AAA testing.',
    packages=find_packages(exclude=['tests', 'tests.*']),
    test_suite='nose.collector',
    include_package_data=True,
    zip_safe=False,
    long_description=codecs.open('README.md', encoding='utf-8').read(),
    install_requires=read_requirements_file('requirements.txt'),
    tests_require=read_requirements_file('dev-requirements.txt'),
    version=VERSION,
    author='AWeber Communications',
    author_email='api@aweber.com',
    entry_points={'console_scripts': []},
    extras_require={'tornado': ['tornado>=3.1']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)
